package com.gic.haoban.manage.web.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


import com.alibaba.fastjson.JSONArray;
import com.gic.api.base.commons.Page;
import com.gic.enterprise.api.dto.GicTreeDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.PageResult;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.DepartmentAddQO;
import com.gic.haoban.manage.web.qo.DepartmentEditQO;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.haoban.manage.web.vo.StoreVo;

@RestController
public class DepartmentContoller extends WebBaseController{
	
	private static Logger logger  = LoggerFactory.getLogger(DepartmentContoller.class);
	@Autowired
	private DepartmentApiService departmentApiService;
	
	@Autowired
	private StoreGroupService storeGroupService;
	@Autowired
	private StoreService storeService;
	@Autowired
	private StaffApiService staffApiService;
	@RequestMapping("department-list")
	public HaobanResponse departmentList(String parentId,BasePageInfo pageInfo,String keyword) {
		LoginVO login = (LoginVO) AuthRequestUtil.getSessionUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		logger.info("【部门列表】wxEnterpriseId={}",wxEnterpriseId);
		
		Page<DepartmentDTO> page = new Page<DepartmentDTO>();
		List<DepartmentDTO> list = new ArrayList<DepartmentDTO>();
		if(StringUtils.isNotBlank(parentId)) {
			page = departmentApiService.pageByParentId(parentId,keyword,pageInfo);
		}else {
			DepartmentDTO dto = departmentApiService.getRootByEnterpriseId(wxEnterpriseId);
			list.add(dto);
			page.setTotalPage(1);
			page.setTotalCount(1);
			page.setResult(list);
		}
		List<DepartmentDTO> result = page.getResult();
		for (DepartmentDTO departmentDTO : result) {
			String chainId = departmentDTO.getChainId();
			List<DepartmentDTO> sonList = departmentApiService.listByChainId(chainId,wxEnterpriseId);
			int staffCount = 0;
			for (DepartmentDTO departmentDTO2 : result) {
				staffCount += staffApiService.countByDepartmentId(departmentDTO2.getDepartmentId());
			}
			departmentDTO.setStaffCount(staffCount);
		}
		PageResult<DepartmentDTO> pageVo = new PageResult<>();
        pageVo.setList(result);
        pageVo.setPageNum(page.getCurrentPage());
        pageVo.setPages(page.getPages());
        pageVo.setPageSize(page.getPageSize());
        pageVo.setTotal(page.getTotalCount());
		return resultResponse(HaoBanErrCode.ERR_1,pageVo);
		
	}
	
	@RequestMapping("department-list-level")
	public HaobanResponse departmentLevelList(String parentId) {
		LoginVO login = (LoginVO) AuthRequestUtil.getSessionUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		logger.info("【部门层级列表】wxEnterpriseId={}",wxEnterpriseId);
		List<DepartmentDTO> list = new ArrayList<DepartmentDTO>();
		if(StringUtils.isNotBlank(parentId)) {
			list = departmentApiService.listByParentId(parentId);
		}else {
			DepartmentDTO dto = departmentApiService.getRootByEnterpriseId(wxEnterpriseId);
			list.add(dto);
		}
		
		return resultResponse(HaoBanErrCode.ERR_1,list);
		
	}
	
	@RequestMapping("department-add")
	public HaobanResponse departmentAdd(String parentId,String departmentName) {
		DepartmentDTO dto = departmentApiService.selectById(parentId);
		if(dto == null || dto.getStatusFlag() == 0) {
			return resultResponse(HaoBanErrCode.ERR_10001);
		}
		if(dto.getIsStore() == 1) {
			return resultResponse(HaoBanErrCode.ERR_10002);
		}
		DepartmentDTO department = new DepartmentDTO();
		department.setParentDepartmentId(parentId);
		department.setDepartmentName(departmentName);
		department.setWxEnterpriseId(dto.getWxEnterpriseId());
		department.setChainId(dto.getChainId() + Constant.ID_SEPARATOR + dto.getDepartmentId());
		department.setIsStore(0);
		department.setLevel(dto.getLevel() + 1);
		departmentApiService.add(department);
	
		return resultResponse(HaoBanErrCode.ERR_1,department);
		
	}
	
	@RequestMapping("department-edit")
	public HaobanResponse departmentEdit(String departmentId,String departmentName) {
		DepartmentDTO dto = departmentApiService.selectById(departmentId);
		if(dto == null || dto.getStatusFlag() == 0) {
			return resultResponse(HaoBanErrCode.ERR_10001);
		}
		dto.setDepartmentName(dto.getDepartmentName());
		departmentApiService.edit(dto);
	
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}
	
	@RequestMapping("department-sel")
	public HaobanResponse departmentSel(String departmentId) {
		DepartmentDTO dto = departmentApiService.selectById(departmentId);
		if(dto == null || dto.getStatusFlag() == 0) {
			return resultResponse(HaoBanErrCode.ERR_10003);
		}
	
		return resultResponse(HaoBanErrCode.ERR_1,dto);
		
	}
	
	@RequestMapping("department-del")
	public HaobanResponse departmentDel(String departmentId) {
		DepartmentDTO dto = departmentApiService.selectById(departmentId);
		if(dto == null || dto.getStatusFlag() == 0) {
			return resultResponse(HaoBanErrCode.ERR_10003);
		}
		HaobanResponse hr;
		if(StringUtils.isBlank(dto.getRelatedId())) {
			hr = departmentApiService.del(departmentId);
		}else {
			hr =  departmentApiService.recycle(departmentId);
		}
		
		if(hr.getErrorCode() == 0) {
			return resultResponse(HaoBanErrCode.ERR_0, hr.getMessage());
		}
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}
	
	@RequestMapping("department-batch-sync")
	public HaobanResponse departmentBatchSync(String editDepartment,String delDepartmentIds,String addDepartment) {
		if(StringUtils.isNotBlank(addDepartment)) {
			List<DepartmentAddQO> list = JSONArray.parseArray(addDepartment, DepartmentAddQO.class);
			for (DepartmentAddQO departmentAddQO : list) {
				String parentId = departmentAddQO.getParentId();
				DepartmentDTO dto = departmentApiService.selectById(parentId);
				if(dto == null || dto.getStatusFlag() == 0) {
					continue;
				}
				if(dto.getIsStore() == 1) {
					continue;
				}
				DepartmentDTO department = new DepartmentDTO();
				department.setParentDepartmentId(parentId);
				department.setWxDepartmentId(dto.getWxDepartmentId());
				department.setDepartmentName(departmentAddQO.getDepartmentName());
				department.setChainId(dto.getChainId() + Constant.ID_SEPARATOR + dto.getDepartmentId());
				department.setChainName(dto.getChainName() + Constant.NAME_SEPARATOR + dto.getDepartmentName());
				department.setIsStore(0);
				department.setIsStore(departmentAddQO.getStoreFlag());
				department.setLevel(dto.getLevel() + 1);
				HaobanResponse hr = departmentApiService.add(department);
				if(hr.getErrorCode() != 1) {
					continue;
				}
				DepartmentDTO departmentAdd = (DepartmentDTO) hr.getResult();
				String departmentId = departmentAdd.getDepartmentId();
				
				String sonDepartment = departmentAddQO.getSonDepartment();
				handler(sonDepartment,departmentId);
				
			}
		}
		
		if(StringUtils.isNotBlank(editDepartment)) {
			List<DepartmentEditQO> list = JSONArray.parseArray(editDepartment, DepartmentEditQO.class);
			for (DepartmentEditQO departmentQO : list) {
				String parentId = departmentQO.getCurrDepartment();
				DepartmentDTO parent = departmentApiService.selectById(parentId);
				if(parent == null || parent.getStatusFlag() == 0) {
					continue;
				}
				DepartmentDTO department = departmentApiService.selectById(parentId);
				department.setParentDepartmentId(parentId);
				department.setDepartmentName(departmentQO.getDepartmentName());
				department.setChainId(parent.getChainId() + Constant.ID_SEPARATOR + parent.getDepartmentId());
				department.setChainName(parent.getChainName() + Constant.NAME_SEPARATOR + parent.getDepartmentName());
				department.setLevel(parent.getLevel() + 1);
				
				handerSonDepartment(departmentQO.getCurrDepartment());
				HaobanResponse hr = departmentApiService.edit(department);
				
			}
			
		}
		
		if(StringUtils.isNotBlank(delDepartmentIds)) {
			String[] delIds = delDepartmentIds.split(",");
			for (String string : delIds) {
				DepartmentDTO dto = departmentApiService.selectById(string);
				if(dto == null || dto.getStatusFlag() == 0) {
					continue;
				}
				if(StringUtils.isBlank(dto.getRelatedId())) {
					departmentApiService.del(string);
				}else {
					departmentApiService.recycle(string);
				}
			}
		}
		
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}
	
	/**
	 * 修改子孙部门
	 * @param departmentId
	 */
	private void handerSonDepartment(String departmentId) {
		DepartmentDTO department = departmentApiService.selectById(departmentId);
		List<DepartmentDTO> list = departmentApiService.listByParentId(departmentId);
		if(!list.isEmpty()) {
			for (DepartmentDTO departmentDTO : list) {
				departmentDTO.setChainId(department.getChainId() + Constant.ID_SEPARATOR + department.getDepartmentId());
				departmentDTO.setChainName(department.getChainName() + Constant.NAME_SEPARATOR + department.getDepartmentName());
				departmentDTO.setLevel(department.getLevel());
				departmentApiService.edit(departmentDTO);
			}
		}
		
		
	}

	@RequestMapping("unbind-department-list")
	public HaobanResponse unbindDepartmentList(String departmentId,String enterpriseId,Integer type) {
		List<String> list = new ArrayList<String>();
		list.add(departmentId);
		List<GicTreeDTO> departmentList = storeGroupService.listGicTree(enterpriseId,type, departmentId);
		
		List<DepartmentDTO> dtoList = new ArrayList<DepartmentDTO>();
		for (GicTreeDTO departmentDTO : departmentList) {
			DepartmentDTO dto = departmentApiService.selectByRelatedId(departmentDTO.getId());
			if(dto == null) {
				DepartmentDTO unBindDTO = new DepartmentDTO();
				unBindDTO.setRelatedId(departmentDTO.getId());
				unBindDTO.setIsStore(departmentDTO.getType());
				unBindDTO.setDepartmentName(departmentDTO.getName());
				dtoList.add(unBindDTO);
			}
		}
		
		return resultResponse(HaoBanErrCode.ERR_1,dtoList);
		
	}
	
	
	@RequestMapping("department-recycle-list")
	public HaobanResponse departmentRecycleList(String keyword,Integer storeFlag,BasePageInfo pageInfo) {
		LoginVO login =  (LoginVO) AuthRequestUtil.getAppLoginUser();
		String wxEnterpriseId  = login.getWxEnterpriseId();
		Integer recycleFlag = 1;
		Page<DepartmentDTO> page = departmentApiService.pageDepartmentByParams(pageInfo,wxEnterpriseId,keyword,storeFlag,recycleFlag);
		PageResult<DepartmentDTO> pageVo = new PageResult<>();
        pageVo.setList(page.getResult());
        pageVo.setPageNum(page.getCurrentPage());
        pageVo.setPages(page.getPages());
        pageVo.setPageSize(page.getPageSize());
        pageVo.setTotal(page.getTotalCount());
        return resultResponse(HaoBanErrCode.ERR_1, pageVo);
		
	}
	
	@RequestMapping("department-repair")
	public HaobanResponse departmentRepair(String departmentId) {
		DepartmentDTO dto = departmentApiService.selectById(departmentId);
		if(dto == null || dto.getStatusFlag() == 0) {
			return resultResponse(HaoBanErrCode.ERR_10003);
		}
		departmentApiService.repairRecycle(departmentId);
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}
	
	
	/**
	 * 处理子节点
	 * @param sonDepartment
	 * @param parentId
	 */
	private void handler(String sonDepartment, String parentId) {
		if(StringUtils.isNotBlank(sonDepartment)) {
			List<DepartmentAddQO> list = JSONArray.parseArray(sonDepartment, DepartmentAddQO.class);
			for (DepartmentAddQO departmentAddQO : list) {
				
				String sonParentId = "";
				DepartmentDTO dto = departmentApiService.selectById(parentId);
				if(dto == null || dto.getStatusFlag() == 0) {
					continue;
				}
				if(dto.getIsStore() == 1) {
					continue;
				}
				String departmentId = departmentAddQO.getDepartmentId();
				//在部门下新增部门
				if(StringUtils.isBlank(departmentId)) {
					DepartmentDTO department = new DepartmentDTO();
					department.setParentDepartmentId(parentId);
					department.setWxEnterpriseId(dto.getWxEnterpriseId());
					department.setDepartmentName(departmentAddQO.getDepartmentName());
					department.setChainId(dto.getChainId() + Constant.ID_SEPARATOR + dto.getDepartmentId());
					department.setChainName(dto.getChainName() + Constant.NAME_SEPARATOR + dto.getDepartmentName());
					department.setIsStore(0);
					department.setIsStore(departmentAddQO.getStoreFlag());
					department.setLevel(dto.getLevel() + 1);
					department.setRelatedId(departmentAddQO.getRelatedId());
					HaobanResponse hr = departmentApiService.add(department);
					if(hr.getErrorCode() != 1) {
						continue;
					}
					DepartmentDTO departmentAdd = (DepartmentDTO) hr.getResult();
					sonParentId = departmentAdd.getDepartmentId();
				//别的部门调整过来的
				}else {
					DepartmentDTO department = departmentApiService.selectById(departmentId);
					if(department == null || department.getStatusFlag() == 0) {
						continue;
					}
					department.setParentDepartmentId(parentId);
					department.setParentDepartmentId(parentId);
					department.setDepartmentName(departmentAddQO.getDepartmentName());
					department.setChainId(dto.getChainId() + Constant.ID_SEPARATOR + dto.getDepartmentId());
					department.setChainName(dto.getChainName() + Constant.NAME_SEPARATOR + dto.getDepartmentName());
					HaobanResponse hr = departmentApiService.edit(department);
					if(hr.getErrorCode() != 1) {
						continue;
					}
					DepartmentDTO departmentEdit = (DepartmentDTO) hr.getResult();
					sonParentId = departmentEdit.getDepartmentId();
				}
				
				sonDepartment = departmentAddQO.getSonDepartment();
				handler(sonDepartment,sonParentId);
				
			}
			
		}
		
	}
	//溢出门店列表
	@IgnoreLogin
	@RequestMapping("store-full-list")
	public HaobanResponse storeFullList(BasePageInfo basePageInfo,String search) {
//		LoginVO login = (LoginVO) AuthRequestUtil.getSessionUser();
//		String wxEnterpriseId = login.getWxEnterpriseId();
		String wxEnterpriseId = "1";
		int maxVersionCount = 0;
		Page<DepartmentDTO> page = departmentApiService.pageFullStoreByWxEnterpriseId(wxEnterpriseId, search,maxVersionCount, basePageInfo);
		List<DepartmentDTO> list = page.getResult();
		List<StoreVo> resultList = EntityUtil.changeEntityListByJSON(StoreVo.class,list);
		
		
		
		List<String> storeIds = list.stream().map(s->s.getRelatedId()).collect(Collectors.toList());
		String [] storeIdArr =  storeIds.toArray(new String[storeIds.size()]);
		List<StoreDTO> storeList = storeService.getStores(storeIdArr);
		Map<String,StoreDTO> map = com.gic.commons.util.CollectionUtil.toMap(storeList, "storeId");
		
		for(StoreVo vo:resultList){
			if(map.get(vo.getRelatedId())!=null){
				vo.setStoreCode(map.get(vo.getDepartmentId()).getStoreCode());
				vo.setStoreId(map.get(vo.getDepartmentId()).getStoreId());
			}
		}
		
		
		Page resultPage = new Page<>(); 
		resultPage.setCurrentPage(page.getCurrentPage());
		resultPage.setPageSize(page.getPageSize());
		resultPage.setTotalCount(page.getTotalCount());
		resultPage.setResult(resultList);
		return resultResponse(HaoBanErrCode.ERR_1,resultPage);
		
		
		
		/*int pageNum = basePageInfo.getPageNum();
		int pageSize = basePageInfo.getPageSize();
		Page<DepartmentDTO> page = new Page<>(pageNum, pageSize, 0);
		
		//TODO　获取版本容量
		int maxSize = 0;
		List<DepartmentDTO> list = departmentApiService.listStoreByWxEnterpriseId(wxEnterpriseId);
		if(list == null||list.size()==0){
			return resultResponse(HaoBanErrCode.ERR_1,new Page<>());
		}
		page.setTotalCount(list.size());
		if(maxSize != 0){
			if(list.size() <= maxSize){
				//小于版本容量（无溢出门店）
				return resultResponse(HaoBanErrCode.ERR_1,new Page<>());
			}else{
				//大于版本容量（有溢出门店）
				list = list.subList(maxSize,list.size()-1);
			}
		}
		List resultList = ListUtils.Pager(pageSize, pageNum, list);
		
		return resultResponse(HaoBanErrCode.ERR_1,resultList);*/
	}
	
}
