package com.gic.haoban.manage.web.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.dubbo.common.URL;
import com.alibaba.fastjson.JSON;
import com.gic.commons.web.ResponseThreadLocal;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;


@RestController
public class LoginController extends WebBaseController{
	
	private static Logger logger= LoggerFactory.getLogger(LoginController.class);
	@Autowired
	StaffApiService staffApiService;
	
	@Autowired
	private QywxCorpApiService qywxCorpApiService;
	
	
	@Autowired
	private Config config;
	
	@RequestMapping("login")
	public void login(String auth_code,HttpServletResponse response) {
		String siteId = config.getSuiteId();
		//String userId = qywxCorpApiService.getUserInfoSimple(siteId,auth_code);
		String userId = qywxCorpApiService.getLoginUserInfo(config.getCorpid(), auth_code);
		logger.info("【企业微信登录】userId={},auth_code={}",userId,auth_code);
		StaffDTO loginStaff = staffApiService.selectByUserId(userId);

		LoginVO loginVO = new LoginVO();
		StaffDTO staff = staffApiService.selectById("1");
		logger.info("【好办登录】staff={}",JSON.toJSONString(staff));
		loginVO.setWxEnterpriseId(staff.getWxEnterpriseId());
		loginVO.setStaffDTO(staff);
		AuthRequestUtil.setSessionUser(loginVO);
		String redirectUri = config.getHost() + "index";
		
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		try {
			response.sendRedirect(redirectUri);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	@RequestMapping("get-login-qrcode")
	public HaobanResponse getLoginQrcode() throws UnsupportedEncodingException {
		String corpid = config.getCorpid();
		String redirectUri = config.getHost() + "login";
		String url = "https://open.work.weixin.qq.com/wwopen/sso/3rd_qrConnect?appid="+corpid+"&redirect_uri="+ URLEncoder.encode(redirectUri, "UTF-8")+"&state=web_login@gyoss9&usertype=admin";
		//String url = qywxCorpApiService.getGrantAuthorizationUrl(siteId, redirectUri);
		return resultResponse(HaoBanErrCode.ERR_1,URL.decode(url));
		
	}
	//运维登录
	@IgnoreLogin
	@RequestMapping("yw-login")
	public void ywLogin(String wxEnterpriseId,HttpServletResponse response) {
		//TODO 获取超级管理员账号接口
		
		StaffDTO staff = staffApiService.selectById("1");
		LoginVO loginVO = new LoginVO();
		loginVO.setWxEnterpriseId(wxEnterpriseId);
		loginVO.setStaffDTO(staff);
		AuthRequestUtil.setSessionUser(loginVO);
		String redirectUri = config.getHost() + "index";
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		try {
			response.sendRedirect(redirectUri);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	}
	
	@RequestMapping("login-out")
	public HaobanResponse loginOut() {
		
		AuthRequestUtil.delSessionUser();
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}

}
